'use strict';


var itxapp = {
	loadedScripts: {},
	config: window.toolboxConfig
};


function startApp() {
	document.addEventListener('DOMContentLoaded', function() {
		loadWidget( 'MenuWidget', '#menuContainer' );
		
		gotoRoute();
	});
	
	
	window.addEventListener("popstate", (evt) => {
		
		// close popup on back?
//		if ($('.popup-element').length > 0) {
//			evt.preventDefault();
//			history.go(1);
//			close_popup();
//			return;
//		}
//		else {
//			// window.location is set at the moment
//			gotoRoute();
//		}
		// window.location is set at the moment
		gotoRoute();
	});
	
	// auto handle
	window.addEventListener('EzTemplate.updated', (evt) => {
		handleAnchors( 'body' );
	});
}


function appUrl( u ) {
	u = itxapp.config.contextPath + u;
	return u;
}



function lookupRoute( u ) {
	let routes = [];
	routes.push({ url: '/', 									widget: 'dashboards/ContractsDashboardWidget' });
	routes.push({ url: '/customers/', 							widget: 'customers/CustomerListWidget' });
	routes.push({ url: '/customers/edit/', 						widget: 'customers/CustomerEditWidget' });
	
	
	routes.push({ url: '/masterdata/activity/', 				widget: 'masterdata/activity/ActivityListWidget' });
	
	routes.push({ url: '/masterdata/users/', 					widget: 'masterdata/users/UserListWidget' });
	routes.push({ url: '/masterdata/users/edit/', 				widget: 'masterdata/users/UserEditWidget' });

	routes.push({ url: '/masterdata/groups/', 					widget: 'masterdata/groups/GroupListWidget' });
	routes.push({ url: '/masterdata/groups/edit/', 				widget: 'masterdata/groups/GroupEditWidget' });
	
	routes.push({ url: '/contracts/', 							widget: 'contracts/ContractListWidget' });
	routes.push({ url: '/contracts/edit/', 						widget: 'contracts/ContractEditWidget' });
	routes.push({ url: '/contracts/tasks/', 					widget: 'contracts/tasks/ContractTaskListWidget' });
	routes.push({ url: '/masterdata/contracts/types/', 			widget: 'contracts/masterdata/ContractTypeListWidget' });
	routes.push({ url: '/masterdata/contracts/types/edit/', 	widget: 'contracts/masterdata/ContractTypeEditWidget' });
	
	routes.push({ url: '/files/', 								widget: 'files/FileListWidget' });
	
	routes.push({ url: '/masterdata/sales/settings/', 			widget: 'sales/SalesSettingsWidget' });
	
	routes.push({ url: '/masterdata/sales/vat/', 				widget: 'sales/vat/VatListWidget' });
	routes.push({ url: '/masterdata/sales/vat/edit/', 			widget: 'sales/vat/VatEditWidget' });
	routes.push({ url: '/masterdata/sales/article/', 			widget: 'sales/article/ArticleListWidget' });
	routes.push({ url: '/masterdata/sales/article/edit/', 		widget: 'sales/article/ArticleEditWidget' });
	routes.push({ url: '/masterdata/sales/articlegroup/', 		widget: 'sales/articlegroup/ArticleGroupListWidget' });
	routes.push({ url: '/masterdata/sales/articlegroup/edit/',	widget: 'sales/articlegroup/ArticleGroupEditWidget' });
	
	routes.push({ url: '/masterdata/sales/slotplan/',			widget: 'sales/slot/SlotplanListWidget' });
	routes.push({ url: '/masterdata/sales/slotplan/config/',	widget: 'sales/slot/SlotplanConfigWidget' });

	routes.push({ url: '/masterdata/sales/pricetype/',			widget: 'sales/pricetype/PricetypeListWidget' });
	routes.push({ url: '/masterdata/sales/pricetype/edit/',		widget: 'sales/pricetype/PricetypeEditWidget' });
	
	routes.push({ url: '/sales/offer/',							widget: 'sales/offer/OfferListWidget' });
	routes.push({ url: '/sales/offer/edit/',					widget: 'sales/offer/OfferEditWidget' });
	
	routes.push({ url: '/sales/invoice/',						widget: 'sales/invoice/InvoiceListWidget' });
	routes.push({ url: '/sales/invoice/edit/',					widget: 'sales/invoice/InvoiceEditWidget' });
	
	routes.push({ url: '/sales/salesplan/',						widget: 'sales/salesplan/SalesPlanWidget' });
	routes.push({ url: '/sales/salesplan/checkout/',			widget: 'sales/salesplan/SalesCheckoutWidget' });
	routes.push({ url: '/sales/salesplan/checkout/articles/',	widget: 'sales/salesplan/SalesCheckoutWidget' });
	routes.push({ url: '/sales/salesplan/checkout/customer/',	widget: 'sales/salesplan/SalesCheckoutWidget' });
	routes.push({ url: '/sales/salesplan/checkout/overview/',	widget: 'sales/salesplan/SalesCheckoutWidget' });
	
	routes.push({ url: '/masterdata/email/settings/',			widget: 'email/EmailSettingsWidget' });
	routes.push({ url: '/email/edit/',							widget: 'email/EmailEditWidget' });
	routes.push({ url: '/email/view/',							widget: 'email/EmailViewWidget' });
	routes.push({ url: '/email/',								widget: 'email/EmailListWidget' });
	
	routes.push({ url: '/masterdata/', 							widget: 'masterdata/MasterDataWidget' });
	routes.push({ url: '/masterdata/appsettings/', 				widget: 'masterdata/AppSettingsWidget' });
	routes.push({ url: '/masterdata/license/',	 				widget: 'masterdata/LicenseWidget' });
	routes.push({ url: '/404/', 								widget: 'NotFoundWidget' });

	for(let i in routes) {
		if ( routes[i].url == u || toolboxConfig.contextPath + routes[i].url == u ) {
			return routes[i];
		}
	}
	console.log('Route not found', u);
	
	return null;
}

function gotoRoute( url ) {
	destroy_popup();
	
	let u;
	
	if (url) {
		u = url;
		
		if (u.indexOf('?') != -1) {
			u = u.substring( 0, u.indexOf('?') );
		}
		
	}
	// determine path, excluding contextPath-url
	else {
		u = window.location.pathname;
		if (itxapp.config.contextPath.length > 0)
			u = u.substring( itxapp.config.contextPath.length );
	}
	
	console.log('loading url: ' + u);
	
	// lookup route
	let r = lookupRoute( u );
	
	// 404?
	if (r == null)
		r = lookupRoute( '/404/' );
	
	
	if (url) {
		window.history.pushState({}, null, url);
	}
	
	$(window).trigger('window-history-pushed');
	
	// show
	loadWidget( r.widget, '#pagecontent' );
	
	setTimeout(function() {
		$(window).scrollTop(0);
	}, 200);
}


function loadWidget( widgetName, container, opts ) {
	if (!container)
		container = '#pagecontent';

	// determine path
	let jsfile = itxapp.config.contextPath + '/static/app/widget/' + widgetName + '.js';

	// import
	import( jsfile ).then (function(mod) {
		let i = jsfile.lastIndexOf('/');
		let widgetName = jsfile.substring(i+1);
		widgetName = widgetName.substring(0, widgetName.length-3);
		
		if ( typeof mod[widgetName] == 'undefined' ) {
			console.error( 'Error: import(), widget not exported, ', jsfile, widgetName );
		}
		
		let w = Reflect.construct( mod[widgetName], [] );
		w.init( container, opts );
	}).catch(function(e) {
		console.log('error loadWidget', e);
		
		showAlert('Error', 'Error: ' + e.message);
	});

}



function showErrorWidget( err ) {
	loadWidget( 'ErrorReportWidget', '#pagecontent', { error: err } );
}


function loadCss( path ) {
	let l = document.createElement('link');
	l.rel = 'stylesheet';
	l.href = appUrl( path );
	
	document.querySelector( 'head' ).appendChild( l );
}




//function loadScript( jsfile, callback ) {
//	itxapp.loadedScripts[jsfile] = 1;
//	
//	let s = document.createElement('script');
//	s.async = false;
//	
//	s.addEventListener('load', function() {
//		if (callback)
//			callback();
//		
//		itxapp.loadedScripts[jsfile] = 2;
//	});
//	
//	s.src = jsfile;
//	
//	document.body.appendChild( s );
//}


/**
 * handleAnchors() - handles <a>-tags
 */
function handleAnchors( e ) {
	
	let el = document.querySelector( e );
	
	let anchs = el.querySelectorAll( 'a' );
	
//	console.log(anchs);
	for(let i=0; i < anchs.length; i++) {
		let a = anchs.item(i);
		
		// already set?
		if (a.linkRouteSet)
			continue;
//		console.log(a);
		
		// href with ':'? => skip
		let href = a.getAttribute('href');
		if (href.indexOf(':') != -1 || href.length == 0)
			continue;
		
		a.addEventListener('click', function(e) {
			e.preventDefault();
			
			gotoRoute( this.getAttribute('href') );
		});
//		a.setAttribute('href', '#');
		
		// mark handled
		a.linkRouteSet = true;
	}
}


// disable submit on return/enter
window.addEventListener('EzTemplate.updated', function() {
	$('form').each(function(index, node) {
		if (node.itxPreventDefaultSet)
			return;
		
		node.itxPreventDefaultSet = true;
		$(node).on('submit', function(evt) {
			evt.preventDefault();
		});
		
		$('.format-price').each(function(index, node) {
			if (node.formatPriceSet) return;
			node.formatPriceSet = true;
			
			$(node).find('input[type=text]').on('change', function() {
				this.value = format_price( this.value, true );
			});
			
		})
		
	});
});


if (!itxapp.config.desktopMode) {
	// non-desktop mode? ping every 10 minuts to refresh auth cookie
	setInterval(function() {
		postUrl( '/service/ping.do' );
	}, 60000 * 10);
}



