
import { EmailPopup } from './EmailPopup.js';
import { PhonePopup } from './PhonePopup.js';
import { AddressPopup } from './AddressPopup.js';
import { NoteListWidget } from '../notes/NoteListWidget.js';


export class CustomerEditWidget {
	
	tpl = `
		<div class="page-header">
			<div class="toolbox">
				<a href="{{appUrl('/customers/')}}" class="fa fa-chevron-circle-left"></a>
				<span ez-if="customer.customerId > 0">
					<a href="javascript:void(0);" [onclick]="widget.deleteCustomer" class="fa fa-trash"></a>
				</span>
				<a href="javascript:void(0);" [onclick]="widget.saveCustomer" class="fa fa-save"></a>
			</div>
			<h1>{{!customer.customerId?'Nieuwe relatie':'Bewerk relatie'}} <span class="title-customer-name"></span></h1>
		</div>
		
		

		<ul class="nav nav-tabs" id="customerTabs" role="tablist">
			<li class="nav-item" role="presentation">
				<button class="nav-link active" id="base-tab" data-bs-toggle="tab" data-bs-target="#base" type="button" role="tab" aria-controls="base" aria-selected="true">Basisgegevens</button>
			</li>
			<li class="nav-item" role="presentation" ez-if="isNew == false">
				<button class="nav-link" id="notes-tab" data-bs-toggle="tab" data-bs-target="#notes" type="button" role="tab" aria-controls="notes" aria-selected="false">Notities</button>
			</li>
			<li class="nav-item" role="presentation" ez-if="isNew == false">
				<button class="nav-link" id="contracts-tab" data-bs-toggle="tab" data-bs-target="#contracts" type="button" role="tab" aria-controls="contracts" aria-selected="false">Contracten</button>
			</li>
		</ul>
		
		<form id="frmCustomer" action="{{appUrl('/service/customers.do')}}">
			<div class="tab-content" id="customerTabsContent">
				<div class="tab-pane fade show active" id="base" role="tabpanel" aria-labelledby="base-tab">
					<input type="hidden" name="a" value="save" />
					<input type="hidden" name="customerId" value="{{customer.customerId}}" />

					<ez-radio name="customerType" label="Type" [value]="customer.customerType">
						<option value="business">Zakelijk</option>
						<option value="personal">Prive</option>
					</ez-radio>

					<ez-text name="customerName" label="Naam" [value]="customer.customerName"></ez-text>

					<ez-text id="contactPerson-container" name="contactPerson" label="Contactpersoon" [value]="customer.contactPerson"></ez-text>

					<ez-text name="cocNumber" label="Kvk nummer" [value]="customer.cocNumber"></ez-text>

					<ez-text name="vatNumber" label="Btw nummer" [value]="customer.vatNumber"></ez-text>

					<ez-text name="iban" label="IBAN" [value]="customer.iban"></ez-text>

					<ez-text name="bic" label="BIC" [value]="customer.bic"></ez-text>

					<ez-textarea name="note" label="Notitie" [value]="customer.note"></ez-textarea>
					
					<ez-datetime-text label="Bewerkt op" hideempty="1" [value]="customer.edited"></ez-datetime-text>
					<ez-datetime-text label="Aangemaakt op" hideempty="1" [value]="customer.created"></ez-datetime-text>
					
					
					<fieldset class="fieldset-email">
						<legend>E-mail</legend>
						<table class="table table-emails table-edit">
							<thead>
								<tr>
									<th></th>
									<th>E-mailadres</th>
									<th>Note</th>
									<th></th>
								</tr>
							</thead>
							<tbody id="tbody-emails" ez-for="customer.emails" ez-item="e">
								<tr [onclick]="widget.editEmail" [data-email]="e" class="pointer">
									<td class="sort-cell" style="width: 20px;"><i class="fa fa-up-down sort-handle ui-sortable-handle" /></td>
									<td>{{e.emailAddress}}</td>
									<td>{{e.description}}</td>
									<td class="actions"><a href="javascript:void(0);" [onclick]="widget.deleteEmail" class="fa fa-trash"></a></td>
								</tr>
							</tbody>
							<tbody class="no-results">
								<tr>
									<td colspan="4">Geen resultaten gevonden</td>
								</tr>
							</tbody>
						</table>

						<a href="javascript:void(0)" [onclick]="widget.addEmail">E-mailadres toevoegen</a>
					</fieldset>
					
					<fieldset class="fieldset-phone">
						<legend>Telefoonnummers</legend>
						
						<table class="table table-phones table-edit">
							<thead>
								<tr>
									<th></th>
									<th>Nummer</th>
									<th>Omschrijving</th>
									<th></th>
								</tr>
							</thead>
							<tbody id="tbody-phones" ez-for="customer.phones" ez-item="p">
								<tr [onclick]="widget.editPhone" [data-phone]="p" class="pointer">
									<td class="sort-cell" style="width: 20px;"><i class="fa fa-up-down sort-handle ui-sortable-handle" /></td>
									<td>{{p.phonenr}}</td>
									<td>{{p.description}}</td>
									<td class="actions"><a href="javascript:void(0);" [onclick]="widget.deletePhone" class="fa fa-trash"></a></td>
								</tr>
							</tbody>
							<tbody class="no-results">
								<tr>
									<td colspan="3">Geen resultaten gevonden</td>
								</tr>
							</tbody>
						</table>

						<a href="javascript:void(0)" [onclick]="widget.addPhone">Telefoonnummer toevoegen</a>
					</fieldset>
					
					<fieldset class="fieldset-address">
						<legend>Adressen</legend>
							<table class="table table-addresses table-edit">
								<thead>
									<tr>
										<th></th>
										<th>Straat</th>
										<th>Postcode</th>
										<th>Plaats</th>
										<th></th>
									</tr>
								</thead>
								<tbody id="tbody-addresses" ez-for="customer.addresses"
									ez-item="a">
									<tr [onclick]="widget.editAddress" [data-address]="a"
										class="pointer">
										<td class="sort-cell" style="width: 20px;"><i class="fa fa-up-down sort-handle ui-sortable-handle" /></td>
										<td>{{a.street ?? ''}} {{a.streetNo ?? ''}}</td>
										<td>{{a.zipcode ?? ''}}</td>
										<td>{{a.city ?? ''}}</td>
										<td class="actions"><a href="javascript:void(0);"
											[onclick]="widget.deleteAddress" class="fa fa-trash"></a></td>
									</tr>
								</tbody>
								<tbody class="no-results">
									<tr>
										<td colspan="6">Geen resultaten gevonden</td>
									</tr>
								</tbody>
							</table>

							<a href="javascript:void(0)" [onclick]="widget.addAddress">Adres toevoegen</a>
					</fieldset>
					
					
				</div>
				
				<div class="tab-pane fade" id="notes" role="tabpanel" aria-labelledby="notes-tab">
					<div id="notes-container"></div>
				</div>
				
				<div class="tab-pane fade" id="contracts" role="tabpanel" aria-labelledby="contracts-tab">
					<div class="action-box">
						<span>
							<a href="javascript:void(0)" [onclick]="widget.addContract">Contract toevoegen</a>
						</span>
					</div>
					
					<br/>

					<table class="table table-contracts">
						<thead>
							<tr>
								<th>Contract</th>
								<th>Waarde</th>
								<th>Contract type</th>
								<th>Startdatum</th>
								<th>Einddatum</th>
								<th>Verlengdatum</th>
								<th>Status</th>
							</tr>
						</thead>
						<tbody ez-for="customer.contracts" ez-item="c">
							<tr [onclick]="widget.editContract" class="pointer"
								contract-id="{{c.contractId}}">
								<td>{{c.name}}</td>
								<td>{{c.amountDescription}}</td>
								<td>{{c.contractTypeName}}</td>
								<td>{{ c.startDate ? format_date(c.startDate, {dmy: true}) : '' }}</td>
								<td>{{ c.endDate ? format_date(c.endDate, {dmy: true}) : ''}}</td>
								<td>{{ c.renewalDate ? format_date(c.renewalDate, {dmy:true}) : ''}}</td>
								<td>{{_('contract_status.'+c.status)}}</td>
							</tr>
						</tbody>
						<tr ez-if="customer.contracts.length == 0">
							<td colspan="8" class="no-results">Geen resultaten gevonden</td>
						</tr>

					</table>
				</div>
			</div>
		</form>
	`;
	
	container = null;
	loading = false;
	
	ezTpl = null;
	isNew = true;
	customer = {
		customerType: 'business',
		customerName: '',
		contactPerson: '',
		phones: [],
		emails: [],
		addresses: []
	};
	
	
	init(e) {
		this.container = e;
		
		let id = getUrlParam('id');
		
		if (id) {
			this.loadCustomer( id );
		}
		else {
			this.render();
		}
	}
	
	render() {
//		console.log( this.tpl );
		this.ezTpl = new EzTemplate( this.container );
		this.ezTpl.loadHtml( this.tpl );
		this.ezTpl.setVar('customer', this.customer);
		this.ezTpl.setVar('isNew', this.isNew);
		this.ezTpl.setObject('widget', this);
		this.ezTpl.render();
		
		$(this.container).find('[name=customerType]').on('change', function() {
			this.toggleCustomerType();
		}.bind(this));
		
		this.toggleCustomerType();
		
		
		$('input[name=customerName]').on( 'change keyup', this.updateTitle );
		this.updateTitle();
		
		$('#tbody-emails').sortable( {
			handle: '.sort-handle'
		});
		
		$('#tbody-phones').sortable( {
			handle: '.sort-handle'
		});
		
		$('#tbody-addresses').sortable( {
			handle: '.sort-handle'
		});
		
		
		focusFirstField('#frmCustomer');
	}
	
	toggleCustomerType() {
		
		let ct = $('input[name=customerType]:checked').val();
		
//		console.log(ct);
		
		if (ct == 'business') {
			document.getElementById('contactPerson-container').style.display = '';
			document.querySelector('ez-text[name=cocNumber]').style.display = '';
			document.querySelector('ez-text[name=vatNumber]').style.display = '';
		}
		else {
			document.getElementById('contactPerson-container').style.display = 'none';
			document.querySelector('ez-text[name=cocNumber]').style.display = 'none';
			document.querySelector('ez-text[name=vatNumber]').style.display = 'none';
		}
	}
	
	updateTitle() {
		
		let customerName = $('#frmCustomer input[name=customerName]').val();
		customerName = $.trim ( customerName );
		
		if (customerName == '') {
			$('.title-customer-name').text('');
		}
		else {
			$('.title-customer-name').text(' - ' + customerName);
		}
	}
	
	
	setCustomer( c ) {
		this.customer = c;
	}
	
	setCustomerName( name ) {
		this.customer.customerName = name;
		
		$(this.container).find('input[name=customerName]').val( name );
	}
	
	
	bindData() {
		this.customer.customerId    = $('input[name=customerId]').val();
		this.customer.customerType  = $('input[name=customerType]:checked').val();
		this.customer.customerName  = $('input[name=customerName]').val();
		this.customer.contactPerson = $('input[name=contactPerson]').val();
		this.customer.cocNumber     = $('input[name=cocNumber]').val();
		this.customer.vatNumber     = $('input[name=vatNumber]').val();
		this.customer.iban          = $('input[name=iban]').val();
		this.customer.bic           = $('input[name=bic]').val();
		this.customer.note          = $('textarea[name=note]').val();
		
		this.customer.emails = new Array();
		this.customer.phones = new Array();
		this.customer.addresses = new Array();
		
		$('#tbody-emails > tr').each(function(index, node) {
			this.customer.emails.push( node['email'] );
		}.bind(this));
		
		$('#tbody-phones > tr').each(function(index, node) {
			this.customer.phones.push( node['phone'] );
		}.bind(this));
		
		$('#tbody-addresses  > tr').each(function(index, node) {
			this.customer.addresses.push( node['address'] );
		}.bind(this));
	}
	
	
	validate( callback_success, callback_error ) {
		showLoadingForm( '#frmCustomer' );
		
		this.bindData();
		
		postUrl( '/service/customers.do'
			, {
				a: 'validateCustomer',
				customer: this.customer
			}).then( async function(e) {
				let json = await e.json();
				hideLoadingForm( '#frmCustomer' );
				
				if (json.error) {
					// render errors
					formShowErrors( '#frmCustomer', json );
					
					if (callback_error)
						callback_error( json );
				}
				else {
					if (callback_success)
						callback_success( json );
				}
				
				this.loading = false;
				
			}.bind(this));
	}
	
	
	saveCustomer() {
		if (this.loading) {
			return;
		}
		
		this.loading = true;
		showLoadingForm( '#frmCustomer' );
		
		this.bindData();
		
		postJson( '/service/customers.do'
			, {
				a: 'save',
				customer: this.customer
			}).then(async function(e) {
				let json = await e.json();
				hideLoadingForm( '#frmCustomer' );
				
				if (json.error) {
					// render errors
					formShowErrors( '#frmCustomer', json );
				}
				else {
					// report succesfully saved
					reportUserMessage( 'Wijzigingen opgeslagen' );
					
					// reload data?
					gotoRoute( appUrl('/customers/edit/?id=' + json.customer.customerId) );
				}
				
				this.loading = false;
				
			}.bind(this));
		
	}
	
	
	getCustomer() {
		this.bindData();
		return this.customer;
	}
	
	loadCustomer(customerId) {
		postUrl( '/service/customers.do'
			, {
				a: 'view',
				customerId: customerId
			}).then(async function(e) {
				let json = await e.json();
				if (json.error) {
					return showErrorWidget(json);
				}
				
				this.customer = json.customer;
				this.isNew = false;
				this.render();
				
				this.loading = false;
				
				this.loadNoteWidget();
			}.bind(this));
	}
	
	deleteCustomer() {
		let id = this.customer.customerId;
		
		
		showConfirmation('Verwijderen', 'Weet je zeker dat je deze relatie wilt verwijderen?', function() {
			postUrl( '/service/customers.do', {
				a: 'delete'
				, 'customerId' : id
			}).then(async function(e) {
				let json = await e.json();
				
				gotoRoute( appUrl('/customers/') );
				reportUserMessage( 'Relatie verwijderd' );
			});
		});
		
	}
	
	
	loadNoteWidget() {
		let notesContainer = document.querySelector('#notes-container');
		
		let w = new NoteListWidget();
		w.setRefObject('customer');
		w.setRefId( this.customer.customerId );
		
		w.init( notesContainer );
	}

	
	addEmail() {
		let e = new EmailPopup();
		
		e.setCallbackSave((p) => {
			let ezTpl = $('#tbody-emails').get(0).eztemplate;
			ezTpl.createRecord( p );
		});
		
		e.showPopup();
	}

	editEmail( p, q ) {
		if ( $(p.target).hasClass('actions') || $(p.target).closest('td.actions').length > 0)
			return;
		
		
		let rec = q['email'];
		
		let popup = new EmailPopup();
		popup.setEmail( rec );
		popup.setCallbackSave((emaildata) => {
			let ezfor = $('#tbody-emails').get(0).eztemplate;
			
			ezfor.updateRecord( q, emaildata );
		});
		popup.showPopup();
		
	}
	
	deleteEmail( evt, rec ) {
		$(rec).closest('tr').remove();
		
		console.log('test', $('#tbody-emails tr').length);
		if ($('#tbody-emails tr').length == 0) {
			$('.table-emails tbody.no-results').removeClass('hidden');
		}
	}
	
	
	
	
	
	addPhone() {
		let p = new PhonePopup();
		
		p.setCallbackSave((p) => {
			let ezTpl = $('#tbody-phones').get(0).eztemplate;
			ezTpl.createRecord( p );
		});
		
		p.showPopup();
	}

	editPhone( p, q ) {
		if ( $(p.target).hasClass('actions') || $(p.target).closest('td.actions').length > 0)
			return;
		
		let rec = q['phone'];
		
		let popup = new PhonePopup();
		popup.setPhone( rec );
		popup.setCallbackSave((phonedata) => {
			let ezfor = $('#tbody-phones').get(0).eztemplate;
			
			ezfor.updateRecord( q, phonedata );
		});
		popup.showPopup();
		
	}
	
	deletePhone( evt, rec ) {
		$(rec).closest('tr').remove();
		
		if ($('#tbody-phones tr').length == 0) {
			$('.table-phones tbody.no-results').removeClass('hidden');
		}
	}
	
	
	
	
	
	addAddress() {
		let e = new AddressPopup();
		
		e.setCallbackSave((p) => {
			let ezTpl = $('#tbody-addresses').get(0).eztemplate;
			ezTpl.createRecord( p );
		});
		
		e.showPopup();
	}

	editAddress( p, q ) {
		if ( $(p.target).hasClass('actions') || $(p.target).closest('td.actions').length > 0)
			return;
		
		
		let rec = q['address'];
		
		let popup = new AddressPopup();
		popup.setAddress( rec );
		popup.setCallbackSave((addressdata) => {
			let ezfor = $('#tbody-addresses').get(0).eztemplate;
			
			ezfor.updateRecord( q, addressdata );
		});
		popup.showPopup();
		
	}
	
	deleteAddress( evt, rec ) {
		$(rec).closest('tr').remove();
		
		if ($('#tbody-addresses tr').length == 0) {
			$('.table-addresses tbody.no-results').removeClass('hidden');
		}
	}
	
	
	editContract(evt) {
		let cid = $(evt.currentTarget).attr('contract-id');
		
		gotoRoute( appUrl('/contracts/edit/?id=' + cid) );
	}
	
	addContract() {
		
		gotoRoute( appUrl('/contracts/edit/?ref_customerId=' + this.customer.customerId) );
		
	}
	
	
}


