


export class UserEditWidget {
	
	tpl = `
		<div class="page-header">
			<div class="toolbox">
				<a href="{{appUrl('/masterdata/users/')}}" class="fa fa-chevron-circle-left"></a>
				
				<span ez-if="user.userId > 0">
					<a href="javascript:void(0);" [onclick]="widget.deleteUser" class="fa fa-trash"></a>
				</span>
				
				<a href="javascript:void(0);" [onclick]="widget.saveUser" class="fa fa-save"></a>
			</div>
			<h1>{{!user.userId?'Nieuwe gebruiker':'Bewerk gebruiker'}}</h1>
		</div>
		
		
		<form id="frmUser" action="{{appUrl('/service/users.do')}}">
		<div class="accordion">
			<div class="accordion-item">
				<h2 class="accordion-header"><button type="button" class="accordion-button hide-button-icon">Basisgegevens</button></h2>
				<div class="accordion-collapse collapse show">
					<div class="accordion-body">
						<input type="hidden" name="a" value="save" />
			
						<input type="hidden" name="user_id" value="{{user.userId}}" />
			
						<ez-checkbox name="activated" label="Actief" [checked]="user.activated"></ez-checkbox>
						
						<ez-select name="userType" label="Gebruikerstype" [value]="user.userType">
							<option value="admin">Administrator</option>
							<option value="user">Gebruiker</option>
						</ez-select>
		
						<ez-text name="username" label="Gebruikersnaam" [value]="user.username"></ez-text>
			
						<ez-password name="password" label="Wachtwoord"></ez-password>
			
						<ez-email name="email" label="E-mail" [value]="user.email"></ez-email>
			
						<ez-text name="firstname" label="Voornaam" [value]="user.firstname"></ez-text>
			
						<ez-text name="lastname" label="Achternaam" [value]="user.lastname"></ez-text>
			
						<ez-datetime-text label="Bewerkt op" hideempty="1" [value]="user.edited"></ez-datetime-text>
			
						<ez-datetime-text label="Aangemaakt op" hideempty="1" [value]="user.created"></ez-datetime-text>
					</div>
				</div>
			</div>
			
			<div class="accordion-item" id="user-group-container" ez-if="groups.length > 0">
				<h2 class="accordion-header"><button type="button" class="accordion-button hide-button-icon">Gekoppelde groepen</button></h2>
				<div class="accordion-collapse show">
					<div class="accordion-body">
						<div ez-for="groups" ez-item="g">
							<div>
								<label>
									<input type="checkbox" [checked]="{{user.userGroupIds && user.userGroupIds.indexOf(g.userGroupId) != -1 ? 1 : 0}}"  id="user-group-id-{{g.userGroupId}}" name="group_id[]" value="{{g.userGroupId}}" />
									{{g.groupName}}
								</label>
							</div>
						</div>
					</div>
				</div>
			</div>
			
		</div>
		</form>
	`;
	
	container = null;
	loading = false;
	
	ezTpl = null;
	groups = [];
	user = {
		activated: true,
		username: ''
	};
	
	
	init(e) {
		this.container = e;
		
		let id = getUrlParam('id');
		
		if (id) {
			this.loadUser( id );
		}
		else {
			this.loadGroups();
			this.render();
		}
	}
	
	render() {
//		console.log( this.tpl );
		this.ezTpl = new EzTemplate( this.container );
		this.ezTpl.loadHtml( this.tpl );
		this.ezTpl.setVar('user', this.user);
		this.ezTpl.setVar('groups', this.groups);
		this.ezTpl.setObject('widget', this);
		this.ezTpl.render();
		
		$(this.container).find('[name=userType]').on('change', function() {
			this.toggleLinkedGroups();
		}.bind(this));
		this.toggleLinkedGroups();
		
		
		if (this.user.userId == 1) {
			$('[name=activated], [name=userType], [name=username]').attr('disabled', 'disabled');
		}
		
	}
	
	toggleLinkedGroups() {
		if ( this.groups.length == 0 )
			return;
		
		let ut = $('select[name=userType]').val();
		
		console.log(ut);
		
		if (ut == 'admin') {
			document.getElementById('user-group-container').style.display='none';
		}
		else {
			document.getElementById('user-group-container').style.display='';
		}
	}
	
	
	saveUser() {
		if (this.loading) {
			return;
		}
		
		this.loading = true;
		
		postForm('#frmUser').then(async function(e) {
			
			let json = await e.json();
			console.log( json );
			
			if (json.error) {
				// TODO: render errors
				formShowErrors( '#frmUser', json );
				
			}
			else {
				// report succesfully saved
				reportUserMessage( 'Wijzigingen opgeslagen' );
				
				// reload data?
				gotoRoute( appUrl('/masterdata/users/edit/?id=' + json.user.userId) );
			}
			
			this.loading = false;
		}.bind(this)).catch(showErrorWidget);
	}
	
	
	deleteUser() {
		showConfirmation('Verwijderen', 'Weet je zeker dat je deze gebruiker wilt verwijderen?', function() {
			postUrl('/service/users.do', {
				a: 'delete',
				userId: this.user.userId
			}).then( async function(e) {
				let json = await e.json();
				if (json.error) {
					showAlert('Error', 'Er is een fout opgetreden: ' + json.message);
					return;
				}
				
				gotoRoute( appUrl('/masterdata/users/') );
			}.bind(this));
		}.bind(this));
	}
	
	loadGroups() {
		postUrl( '/service/users.do', {
			a: 'groups'
		}).then( async function(e) {
			let json = await e.json();
			if (json.error) {
				return showErrorWidget(json);
			}
			
			this.groups  = json;
			this.render();
			
			this.loading = false;
		}.bind(this) );
	}
	
	loadUser(userId) {
		postUrl( '/service/users.do', {
			a: 'view',
			userId: userId
		}).then( async function(e) {
			let json = await e.json();
			if (json.error) {
				return showErrorWidget(json);
			}
			
			this.user = json.user;
			this.groups  = json.groups;
			this.render();
			
			this.loading = false;
		}.bind(this) );
	}
	
}


